from grapejuice import __version__ as gj_package_version

project_name = "Grapejuice"
package_name = "grapejuice"
package_version = gj_package_version
package_license = "GPL-3"
package_platform = "Linux"
package_description = "A simple wine+roblox management application"
package_repository = "https://gitlab.com/brinkervii/grapejuice"
author_name = "BrinkerVII"
author_email = "brinkervii@gmail.com"
